@@ -97,7 +97,7 @@
 #include "arrow/util/checked_cast.h"
 #include "arrow/util/config.h"
 #include "arrow/util/io_util.h"
-#include "arrow/util/logging.h"
+#include "arrow/util/logging_internal.h"
 #include "arrow/util/mutex.h"
 
 // For filename conversion
@@ -118,7 +118,7 @@
 #endif
 
 #ifdef _WIN32
-#  include <Windows.h>
+#  include <windows.h>
 #else
 #  include <dlfcn.h>
 #endif
@@ -2232,7 +2232,7 @@ Result<void*> LoadDynamicLibrary(const char* path) {
   constexpr int kFlags =
       // All undefined symbols in the shared object are resolved before dlopen() returns.
       RTLD_NOW
-      // Symbols defined in  this  shared  object are not made available to
+      // Symbols defined in this shared object are not made available to
       // resolve references in subsequently loaded shared objects.
       | RTLD_LOCAL;
   if (void* handle = dlopen(path, kFlags)) return handle;
